"""Map some Orchestrator types to scalars."""

from ipaddress import IPv4Network, IPv6Network
from typing import Any, NewType

import strawberry
from orchestrator.graphql.types import serialize_to_string
from strawberry.custom_scalar import ScalarDefinition, ScalarWrapper

IPv4NetworkType = strawberry.scalar(
    NewType("IPv4NetworkType", str),
    description="Represent the Orchestrator IPv4Network data type",
    serialize=serialize_to_string,
    parse_value=lambda v: v,
)

IPv6NetworkType = strawberry.scalar(
    NewType("IPv6NetworkType", str),
    description="Represent the Orchestrator IPv6Network data type",
    serialize=serialize_to_string,
    parse_value=lambda v: v,
)

GSO_SCALAR_OVERRIDES: dict[object, Any | ScalarWrapper | ScalarDefinition] = {
    IPv4Network: IPv4NetworkType,
    IPv6Network: IPv6NetworkType,
}
