from unittest.mock import patch

import pytest

from gso.products.product_blocks.router import RouterRole
from gso.products.product_types.iptrunk import Iptrunk
from gso.settings import load_oss_params
from test.services.conftest import MockedNetboxClient
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
@patch("gso.workflows.iptrunk.terminate_iptrunk.infoblox.delete_network")
@patch("gso.services.netbox_client.NetboxClient.delete_interface")
@patch("gso.services.netbox_client.NetboxClient.free_interface")
def test_successful_iptrunk_termination(
    mocked_free_interface,
    mocked_delete_interface,
    mock_infoblox_delete_network,
    mock_execute_playbook,
    iptrunk_subscription_factory,
    faker,
    data_config_filename,
    router_subscription_factory,
):
    #  Set up mock return values
    product_id = iptrunk_subscription_factory()
    mocked_netbox = MockedNetboxClient()
    mocked_delete_interface.return_value = mocked_netbox.delete_interface()
    mocked_free_interface.return_value = mocked_netbox.free_interface()
    #  Add two more routers to our fake network
    router_subscription_factory(router_role=RouterRole.P)
    router_subscription_factory(router_role=RouterRole.PE)

    #  Run workflow
    oss_params = load_oss_params()
    initial_iptrunk_data = [
        {"subscription_id": product_id},
        {
            "tt_number": faker.tt_number(),
            "remove_configuration": True,
        },
    ]
    result, process_stat, step_log = run_workflow("terminate_iptrunk", initial_iptrunk_data)

    for _ in range(3):
        result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)

    # Check NetboxClient calls
    assert mocked_delete_interface.call_count == 2  # once for each side
    assert mocked_free_interface.call_count == 4  # Free interfaces for each side (2 per side)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Iptrunk.from_subscription(subscription_id)

    assert subscription.status == "terminated"
    assert mock_execute_playbook.call_count == 3
    assert mock_infoblox_delete_network.call_count == 2
    assert subscription.iptrunk.iptrunk_isis_metric == oss_params.GENERAL.isis_high_metric
